#include <curses.h>

#ifdef NOPANEL
#include <nopanel.h>
#else
#include <panel.h>
#endif

#include <string.h>
#include "resource.h"

WINDOW *window;
PANEL  *panel;
FINFO  *daten;
char fname[80], expr[80];
int ymax, xmax;

int getcval (char *string)
{ return 0; }

void LoadExternResource (FILE *datei, char *ptr, char *rname, char *rtype)
{}

void InitScreen(void);
void FinishScreen(void);

void KillEndSpaces (char *str)
{
  char *ptr = str + strlen (str) - 1;

  while (ptr != str && *ptr == ' ')
    ptr--;

  if (ptr == str && *ptr==' ')
    *ptr = '\0';
  else
    *++ptr = '\0';
}

void DisplayLines (FILE *input)
{
  char string[255];
  int y;

  if (feof (input))
    return;

  for (y=0; y<ymax; y++) {
    if (fgets (string, 255, input) == NULL) {
      wfillrgn (window, y, 0, ymax, xmax);
      return;
    }
    wfillrgn  (window, y, 0, y, xmax);
    wprintstr (window, string, y, 0);
  }
}

void Main_Program (void)
{
  int res, flag, nlines, ncols;
  FILE *pipe = NULL;
  char comm[256];
  FINFO *ask;

  do {
    flag = 0;

    if (HavetoResize()) {
      FinishScreen ();
      if (GetWinSize (&nlines, &ncols)) {
        LINES = nlines;
        COLS  = ncols;
      }
      DoResize   (LINES, COLS);
      InitScreen ();
    }

    SetHelpContext  ("TheHelp");
    res = InputChar ();
    DelHelpContext  ();

    if (res == '\x1b') { /* Escape is Menu Key */
      res = DoMenu ("MainMenu");

      if (res == -1)
	continue;

      switch (res) {
      case 101: /* Ende */
	if (pipe)
	  pclose (pipe);
	return ;
	break;
      case 110: /* Neue Datei */
	if (strlen (expr) == 0) {
	  InfoUser (GetMsgResource ("NotPossible"), GetMsgResource ("NoExpr"));
	  flag = 1;
	  break;
	}
	ask = InitForm ("GetFileName");
	PutForm (ask, 600, fname);
	res = QueryForm (ask);

	if (res == IT_ESCAPE)
	  flag = 1;
	else
	  KillEndSpaces (strcpy (fname, GetForm (ask, 600)));

	DeleteForm (ask);
	break;
      case 111: /* Neue Expr */
	if (strlen (fname) == 0) {
	  InfoUser (GetMsgResource ("NotPossible"), GetMsgResource ("NoFile"));
	  flag = 1;
	  break;
	}
	ask = InitForm ("GetGrepExpr");
	PutForm (ask, 601, expr);
	res = QueryForm (ask);

	if (res == IT_ESCAPE)
	  flag = 1;
	else
	  KillEndSpaces (strcpy (expr, GetForm (ask, 601)));

	DeleteForm (ask);
	break;
      case 112: /* beides neu */
	ask = InitForm ("GetFileName");
	PutForm (ask, 600, fname);
	res = QueryForm (ask);

	if (res == IT_ESCAPE)
	  flag = 1;
	else
	  KillEndSpaces (strcpy (fname, GetForm (ask, 600)));
	DeleteForm (ask);

	if (!flag) {
	  ask = InitForm ("GetGrepExpr");
	  PutForm (ask, 601, expr);
	  res = QueryForm (ask);

	  if (res == IT_ESCAPE)
	    flag = 1;
	  else
	    KillEndSpaces (strcpy (expr, GetForm (ask, 601)));
	  DeleteForm (ask);
	}
	break;
      }
      if (flag)
	continue;
      
      if (pipe)
	pclose  (pipe);
      PutForm (daten, 600, fname);
      PutForm (daten, 601, expr);
      sprintf (comm, "grep '%s' %s", expr, fname);
      if ((pipe = popen (comm, "r")) == NULL)
	wfill (window);
    }
    else if (res == KEY_F(0)) {
      if (pipe)
	pclose (pipe);
      return ;
    }
    if (pipe)
      DisplayLines (pipe);
  }
  while (TRUE);
}

void InitScreen (void)
{
  InitMenu    ("MainMenu");
  daten = InitForm ("MainInfo");
  DisplayForm (daten);
  wattrset    (stdscr, COLOR_PAIR (1));
  wbox        (stdscr, 6, 0, LINES-1, COLS-1);

  if ((window = newwin (LINES-8, COLS-2, 7, 1)) == NULL) {
    DeleteForm (daten);
    EndCurses  ();
    exit(1);
  }
  if ((panel = new_panel (window)) == NULL) {
    delwin     (window);
    DeleteForm (daten);
    EndCurses  ();
    exit(1);
  }
  getmaxyx (window, ymax, xmax);

  SetKeyHook (CTRL('W'), RedrawScreen);
  SetKeyAlias(CTRL('X'), KEY_F(0));
  SetKeyAlias(KEY_F(10), KEY_F(0));
}

void FinishScreen(void)
{
  DelKeyAlias(KEY_F(10));
  DelKeyAlias(CTRL('X'));
  DelKeyHook (CTRL('W'));

  del_panel  (panel);
  delwin     (window);
  DeleteForm (daten);

  wattrset   (stdscr, COLOR_PAIR(0));
  wfill      (stdscr);
}

int main (void)
{
  if (StartCurses("bemogrep.res", NULL, 0) == FALSE) {
    puts ("Fehler! Resourcedatei nicht gefunden");
    exit(1);
  }
  if (LINES < 24 || COLS < 24) {
    EndCurses();
    puts ("Der Bildschirm ist zu klein fuer dieses Programm");
    exit(1);
  }
  fname[0] = expr[0] = '\0';

  InitScreen   ();
  Main_Program ();
  FinishScreen ();
  EndCurses    ();
}
